function [l,s]=dellqr(a,b,q,r,delta,n)
%DELLQR	    Linear quadratic regulator design for discrete time delta systems.
%	        [L,S] = DELLQR(A,B,Q,R,delta,N)  calculates the optimal feedback 
%           gain matrix L such that the feedback law:  
%                u = -Lx  
%
%           minimizes the cost function:
%                J = x'x + Integral{x'Qx + u'Ru}dt
%
%	        subject to the constraint equation: 
%  		         px = Ax + Bu 
%
%           The solution returned is given
%                                  -1
%                L = (R + deltaB'SB)  B'S(I+Adelta)
%                
%	        Where S is the steady state solution to the associated 
%    	    algebraic Riccati equation:				 
%		         0 = Q + SA + A'S + deltaA'SA - L'(R+deltaB'SB)L
%
%	        N is the cross term that relates u to x in the cost function and 
%           is optional. 


%           P. THIND
%           10/89.
    
    nargs = nargin;
    error(nargchk(5,6,nargin));
    error(abcdchk(a,b));

    [ra,ca] = size(a);
    [rb,cb] = size(b);
    [rq,cq] = size(q);
    if (ra ~= rq) | (ca ~= cq) 
	    error('A and Q must be the same size')
    end
    [rr,cr] = size(r);
    if (rr ~= cr) | (cb ~= rr)
	    error('B and R must be consistent')
    end

    if nargin == 6
	    [r_n,c_n] = size(n);
	    if (r_n ~= ra) | (c_n ~= cr)
		    error('N must be consistent with Q and R')
	    end
	    % Add cross term
	    q = q - n/r*n';
	    a = a - b/r*n';
    else
	    n = zeros(ra,cb);
    end

    % Check if q is positive semi-definite and symmetric
    if any(eig(q) < 0) | (norm(q'-q,1)/norm(q,1) > eps)
	    error('Q must be symmetric and positive semi-definite')
    end

    % Check if r is positive definite and symmetric
    if any(eig(r) <= 0) | (norm(r'-r,1)/norm(r,1) > eps)
	    error('R must be symmetric and positive definite')
    end

    % Start eigenvector decomposition by finding eigenvectors of Hamiltonian M
     
    I = eye(ra,ca);

    M = [I delta*b/r*b';zeros(ra,ca), (I+delta*a')]; 
    M_continuous = [a , -b/r*b'; -q, -a'];

% Don't do this:        M = M\M_continuous;  
% The next line was:    [v,d] = eig(M);
    [v,d]=eig(M_continuous,M);
    d = diag(d);

    % Extra line to avoid problems with NaN that may result if
    % I + delta*a  is singular ( and hence M is singular )
    for ii=1:length(d);
        if isnan(d(ii));
            d(ii)=1e+10;
        end;
    end;

    d = (delta/2)*abs(d).^2 + real(d);
    [d,index] = sort(real(d));                   % sort eigenvalues
    if (~( (d(ca)<0) & (d(ca+1)>0) ))
	    error('Can''t order eigenvalues, (A,B) may be uncontrollable.')
    end

    chi = v(1:ca,index(1:ca));               	  
    lambda = v((ca+1):(2*ca),index(1:ca));        

    s = real(lambda/chi);   

%   This is wrong GvW
%    l = (r + delta*b'*s*b)\(n' + b'*s*(I + a*delta));
%   Instead
    l = (r + delta*b'*s*b)\(b'*s*(I + a*delta))+r\n';
