% DPCFOTIN: Continuous-time Deterministic Parameter Full-Order Time Invariant
%           iNfinite horizon LQG compensation.
%           Optimal compensation of (P,G,C,V,W) based on (Q,R) nc=nx.
%           (P,G,C) deterministic.
%
%           [fo,ko,lo,sigp,sigs,spr,pt,st,sigc]=dpcfotin(p,g,c,v,w,q,r);
% or
%           [fo,ko,lo,sigp,sigs,spr,pt,st,sigc]=dpcfotin(func);
%
% Input:
%          func  : string containing the function name of the function that
%                  specifies all estimator problem parameters (see e.g. cofu).
% Output:
%          fo, ko, lo : Optimal compensator
%          sigp, sigs,
%                sigc : Minimum costs (if sigs=sigp)
%          spr        : Spectral radius close loop system.
%          pt, st     : Solutions estimator and control Riccati equations.
%
%          L.G. Van Willigenburg, W.L. De Koning, 28-11-95.
%

  function [fo,ko,lo,sigp,sigs,spr,pt,st,sigc]=dpcfotin(p,g,c,v,w,q,r);

%
% Get problem data from func=p if p is a string
%
  if isstr(p);
    func=p; [p,g,c,v,w,q,r]=feval(func);
  end;
%
% Check dimensions
%
  [nx,ny,nu,q,mc,v,me]=pgcchk(p,g,c,v,w,q,r);
  [nx1,nx2]=size(q);
  if nx1~=nx | nx2~=nx; error('q incompatible with p'); end;
  [nu1,nu2]=size(r);
  if nu1~=nu | nu2~=nu; error('r incompatible with g'); end;
%
% Compute full-order optimal feedback
%
  [lo,st,e] = lqr(p,g,q,r,mc);
%
% Compute full-order optimal estimator
%
  [ko,pt,e] = lqr(p',c',v,w,me);
%
% Determine full-order optimal compensator and costs
%
  ko=ko'; fo=p-g*lo-ko*c;

  sigs=trace(v*st+lo'*r*lo*pt);
  sigp=trace(q*pt+ko*w*ko'*st);

  pa=[p -g*lo ; ko*c fo]; spr=max(real(eig(pa)));

  if spr>0; sigc=Inf; spr=maxreig(p); fo=[]; ko=[]; lo=[];
  elseif nargout==9
    qa=[q -mc*lo ; -lo'*mc' lo'*r*lo ];
    va=[v  me*ko';  ko *me' ko *w*ko'];
    pp=lyap(pa,va) ; sigc=trace(qa*pp);
%    ss=lyap(pa',qa); sigc=trace(va*ss);
  end
