% RCCHK: Check reduced-order algorithm convergence
%
%          [osc,endc,conver,bb,sigp,sigs,tpst,tsprm,b,iti,dt]=...
%           rcchk(opt,osc,conver,bb,it,itm,itc,pt,st,...
%                 tpst,tsprm,tpsf,epsl,maxtps,fnam,nx,nc,b,a,cc,iti,tpsf,delta,dt);
%
% Input/output arguments:
%
% osc    : oscillation flag and divergence counter
% endc   : flag indicating end of iterations
% conver : number of successive converged iterations
% bb     : adjusted numerical damping
% nocon  : flag indicating convergence status
%          0 standard output
%          1 no convergence
%          2 numerical damping adapted
% tps    : Trace(pt+st)
% tpst   : array of Trace(pt+st) values
% epsl   : epsilon that determines convergence,
%          if length > 1 minimal spectral radius convergence 
% it     : total number of iterations
% iti    : number of iterations after change numerical damping
% itm    : maximum number of iterations
% itc    : plot parameter 1,2 or 3
% tpso   : previous value Trace(pt+st)
% maxtps : Trace(pt+st) > maxtps then divergence
% fnam   : String containing the calling function name
% nx, nc : system and compensator dimension
% b, a   : numerical damping and homotopy parameter
% cc     : flag for automatic adjustment numerical damping
%          0 automatic adjustment
%          1 no automatic adjustment

  function [osc,endc,conver,bb,nocon,tpst,b,iti,dt,idt,rdk,rdl]=rcchk...
  (opt,osc,conver,bb,it,itm,itc,tps,tpst,epsl,maxtps,fnam,...
   plstr,nx,nc,b,a,cc,iti,delta,dt,idt,ndt,dtr,rdk,rdl,ny,nu)

  if nargin<25; ny=nx; nu=nx; rdk=ny; rdl=nu; end
  if opt(10)~=0 && delta==0; itosc=1000; else itosc=100; end
  if conver==-1; nocon=1; endc=1; osc=0;
  elseif idt>ndt
    disp(['  Termination: dt reduced more than ' num2str(ndt) ' times']);
    conver=0; endc=1; nocon=1; osc=0;
  else

  % Compute convergence criterion & append trace array
    if length(tps)==3 % discrete-time case
      if opt(10)==0; tpsr=abs(tps(2)); tpst=[tpst tps(1)];
      elseif it>2; tpsr=abs(tps(3)-tpst(end)); tpst=[tpst tps(3)];
      else tpsr=1; tpst=[tpst tps(3)];
      end
    else % delta operator case
      if opt(10)==0; tpsr=abs(tps(2)); tpst=[tpst tps(1)];
      elseif it>1; tpsr=abs(tps(2)-tpst(end)); tpst=[tpst tps(2)];
      else tpsr=1; tpst=[tpst tps(2)];
      end
    end

    nocon=0; endc=0;
  % Maximum number of iterations exceeded
    if it>itm;
      disp(' ');
      disp(['  ' fnam ' : Maximum number of iterations exceeded']);
      endc=1; nocon=1;

  % Trace exceeds maximum value
    elseif (opt(10)==0 && (abs(tps(1))> maxtps || tps(1) < 0)) || conver==-2;
      conver=0;
      if ~cc
        b=b+0.5*(1-b); iti=0; nocon=2;
        rdk=ny; rdl=nu; dt=dtr^(-idt)*dt;
        disp(' ');
        disp(['  ' fnam ': Divergence'])
        disp(['  Numerical damping increased to: ' num2str(b)]);
        if idt>0; disp(['  dt reset to: ' num2str(dt)]); end
        idt=0;
%         if nargin>=21 && delta == 0 && idt<ndt;
%           dt=dtr*dt; idt=idt+1;
%           disp(['  Integration stepsize dt reduced to: ' num2str(dt)]);
%         end
      end;
      if cc || b >= 0.995
        disp(' ');
        disp(['  ' fnam ' : No convergence']);
        endc=1; nocon=1;
      end

  % Convergence criteria && counts
    elseif tpsr<epsl; conver=conver+1;

  % Check oscillation and bad convergence
    elseif ~cc && rem(iti,itosc)==0;
      if idt>0 && dtr~=1;
        idt=idt-1; dt=dt/dtr;
        disp(' '); disp(['  Integration stepsize dt increased to: ' num2str(dt)]);                              
      end
      if delta>=dt
        [osc]=detosc(osc,tpst,min(round(iti/2),500),min(round(iti/10),100));
      else
        [osc]=detosc(osc,tpst,min(round(iti/2),5e3),min(round(iti/10),1e3),0.8);
      end
      if osc>1
        b=b+0.5*(1-b); iti=0; osc=0; nocon=2;
        rdk=ny; rdl=nu; dt=dtr^(-idt)*dt;
        disp(' ');
        disp(['  ' fnam ': Bad convergence'])
        disp(['  Numerical damping increased to: ' num2str(b)]);
        if idt>0; disp(['  dt reset to: ' num2str(dt)]); end
        idt=0;
%         if length(tps)==2 && idt<ndt
%           dt=dtr*dt; idt=idt+1;
%           disp(' '); disp(['  Integration stepsize dt reduced to: ' num2str(dt)]);
%         end
      end
      if b >= 0.995
        disp(' ');
        disp('  Termination: Numerical damping too large');
        disp(['  ' fnam ' : No convergence']);
        endc=1; nocon=1;
      end
      
  % Nothing special: continu iteration
    else conver=0;
    end

  % Six consequtive iterations converged: convergence
    if conver==max(10,ceil(iti/50)); endc=1;

  % Nothing special: apply current numerical damping
    else bb=b;
    end;

  % Display progress computation every itc iterations.
    if (itc>0 && rem(it,itc)==0) || (itc~=0 && endc==1)
      if a<0; str=['  ' fnam];
      else str=['  ' fnam ' alpha=' num2str(a)]; end
      plconv(tpst,tpsr,epsl,endc,str,plstr,nx,nc);
    end
  end